@@ -13,7 +13,7 @@
 #   Craig Barratt  <cbarratt@users.sourceforge.net>
 #
 # COPYRIGHT
-#   Copyright (C) 2002-2017  Craig Barratt
+#   Copyright (C) 2002-2019  Craig Barratt
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@
 #
 #========================================================================
 #
-# Version 4.1.3, released 3 Jun 2017.
+# Version 4.3.1, released 14 Jul 2019.
 #
 # See http://backuppc.sourceforge.net.
 #
@@ -127,23 +127,22 @@ sub start
                             if ( ref($conf->{RsyncSshArgs}) eq 'ARRAY' );
             push(@$rsyncArgs, @$srcList, "$t->{hostIP}:$remoteDir");
         } else {
-            my $pwFd;
-            $t->{pwFile} = "$conf->{TopDir}/pc/$t->{client}/.rsyncdpw$$";
-            if ( !length($conf->{RsyncdPasswd}) ) {
-                $t->{XferLOG}->write(\"\$Conf{RsyncdPasswd} is empty; host's rsyncd auth will fail\n");
-                $t->{_errStr} = "\$Conf{RsyncdPasswd} is empty; host's rsyncd auth will fail";
-                return;
-            }
-            if ( open($pwFd, ">", $t->{pwFile}) ) {
-                chmod(0400, $t->{pwFile});
-                binmode($pwFd);
-                syswrite($pwFd, $conf->{RsyncdPasswd});
-                close($pwFd);
-                push(@$rsyncArgs, "--password-file=$t->{pwFile}");
-            } else {
-                $t->{XferLOG}->write(\"Failed to open/create rsynd pw file $t->{pwFile}\n");
-                $t->{_errStr} = "Failed to open/create rsynd pw file $t->{pwFile}";
-                return;
+            if ( length($conf->{RsyncdPasswd}) ) {
+                my($pwFd, $ok);
+                $t->{pwFile} = "$conf->{TopDir}/pc/$t->{client}/.rsyncdpw$$";
+                if ( open($pwFd, ">", $t->{pwFile}) ) {
+                    $ok = 1;
+                    $ok = 0 if ( $ok && chmod(0400, $t->{pwFile}) != 1 );
+                    $ok = 0 if ( $ok && !binmode($pwFd) );
+                    $ok = 0 if ( $ok && syswrite($pwFd, $conf->{RsyncdPasswd}) != length($conf->{RsyncdPasswd}) );
+                    $ok = 0 if ( $ok && !close($pwFd) );
+                    push(@$rsyncArgs, "--password-file=$t->{pwFile}");
+                }
+                if ( !$ok ) {
+                    $t->{XferLOG}->write(\"Failed to open/create rsynd pw file $t->{pwFile} ($!)\n");
+                    $t->{_errStr} = "Failed to open/create rsynd pw file $t->{pwFile} ($!)";
+                    return;
+                }
             }
             my $shareName = $t->{shareName};
             #from_to($shareName, "utf8", $conf->{ClientCharset})
@@ -166,6 +165,7 @@ sub start
                             host      => $t->{host},
                             hostIP    => $t->{hostIP},
                             client    => $t->{client},
+			    shareName => $t->{shareName},
                             confDir   => $conf->{ConfDir},
                             sshPath   => $conf->{SshPath},
                         });
@@ -357,6 +357,7 @@ sub start
                             host      => $t->{host},
                             hostIP    => $t->{hostIP},
                             client    => $t->{client},
+			    shareName => $t->{shareName},
                             confDir   => $conf->{ConfDir},
                             sshPath   => $conf->{SshPath},
                         });
@@ -544,6 +545,12 @@ sub run
                 }
                 $t->{stats}{xferErrs}++;
             }
+            if ( /^rsync error: / || /^rsync warning: / ) {
+                $t->{stats}{xferErrs}++;
+            }
+            if ( /^rsync: send_files failed to open / || /^file has vanished: / ) {
+                $t->{stats}{xferErrs}++;
+            }
             if ( /^IOrename:\s(\d+)\s(.*)/ ) {
                 my $oldName = substr($2, 0, $1);
                 my $newName = substr($2, $1);
@@ -603,12 +610,19 @@ sub run
     #
     # Remove any rsyncTmp files in the backup directory
     #
-    my $bkupDir = "$conf->{TopDir}/pc/$t->{client}/$t->{backups}[$t->{newBkupIdx}]{num}";
-    foreach my $e ( @{BackupPC::DirOps::dirRead($bpc, $bkupDir)} ) {
-	if ( $e->{name} =~ /^rsyncTmp\.\d+\.[.\d]+$/ ) {
-            $t->{XferLOG}->write(\"Removing rsync temporary file $bkupDir/$e->{name}\n");
+    my $bkupDir = $t->{type} eq "restore" ? "$conf->{TopDir}/pc/$t->{bkupSrcHost}/$t->{bkupSrcNum}"
+					  : "$conf->{TopDir}/pc/$t->{client}/$t->{backups}[$t->{newBkupIdx}]{num}";
+    my $bkupDirEntries = BackupPC::DirOps::dirRead($bpc, $bkupDir);
+    my $pidRunning = {};
+    if ( ref($bkupDirEntries) eq 'ARRAY' ) {
+        foreach my $e ( @$bkupDirEntries ) {
+            next if ( $e->{name} !~ /^rsyncTmp\.(\d+)\.\d+\.\d+$/ );
+            my $pid = $1;
+            $pidRunning->{$pid} = kill(0, $pid) ? 1 : 0 if ( !defined($pidRunning->{$pid} ) );
+            next if ( $pidRunning->{$pid} );
+	    $t->{XferLOG}->write(\"Removing rsync temporary file $bkupDir/$e->{name}\n");
 	    unlink("$bkupDir/$e->{name}");
-	}
+        }
     }
 
     if ( $t->{type} eq "restore" ) {
